import torch
import torch.nn as nn
import numpy as np

import torch
import torch.optim as optim
import torch.nn.functional as F

from blitz.modules import BayesianLSTM
from blitz.utils import variational_estimator
from torch.autograd import Variable






class Deep_NMT(nn.Module):
    def __init__(self, d_input: int, d_model: int, dropout: float):
        super(Deep_NMT, self).__init__()
        # encoder应该包含label的序列
        self.fc1 = nn.Linear(d_input, d_model)
        self.relu1 = nn.ReLU()
        self.dropout = nn.Dropout(dropout)
        self.fc2 = nn.Linear(d_model, d_model)
        self.relu2 = nn.ReLU()
        self.fc3 = nn.Linear(d_model, 1)

    def forward(self, source_data):
        source_data = source_data.float()
        x = self.fc1(source_data)
        x = self.dropout(x)
        x = self.relu1(x)
        x = self.fc2(x)
        x = self.relu2(x)
        outs = self.fc3(x)


        return outs


@variational_estimator
class NN(nn.Module):
    def __init__(self):
        super(NN, self).__init__()
        self.lstm_1 = BayesianLSTM(15, 128)
        self.linear = nn.Linear(132, 24)

    def forward(self, x, addition_x):
        x_, _ = self.lstm_1(x)

        #gathering only the latent end-of-sequence for the linear layer
        x_ = x_[:, -1, :]

        output = self.linear(torch.cat((x_, addition_x.squeeze(dim=1), 1)))
        return output


class Att_NMT(nn.Module):
    def __init__(self, feature_size, addition_feature_size, output_dim, encoder_length, decoder_length, lstm_size, training=True):
        super(Att_NMT, self).__init__()
        # encoder应该包含label的序列
        # 作为修改，此处尝试双向LSTM
        self.encoder = nn.LSTM(input_size=feature_size + 1, hidden_size=lstm_size, num_layers=1, bidirectional=False,
                               batch_first=True, dropout=0.1)  # if batch_first==False: input_shape=[length,batch_size,embedding_size]
        # decoder的inputsize需要变化，加上2倍的LSTM_size，因为attention的维度就是这个
        self.decoder = nn.LSTM(input_size=addition_feature_size, hidden_size=lstm_size, num_layers=1,
                               batch_first=True, dropout=0.1)
        # attention的全连接
        self.attention_fc_1 = nn.Linear(2 * lstm_size, 2 * lstm_size)
        self.attention_fc_2 = nn.Linear(2 * lstm_size, 1)
        # 最后预测的FC的维度也需要修改

        self.fc_1 = nn.Linear(addition_feature_size + lstm_size, 2 * lstm_size)
        self.fc_2 = nn.Linear(2 * lstm_size, output_dim)
        self.training = training
        self.encoder_length = encoder_length
        self.decoder_length = decoder_length

    def attention_forward(self, source_data, dec_prev_hidden, enc_output):
        prev_dec_h = dec_prev_hidden[0].squeeze(dim=0).unsqueeze(1).repeat(1, self.encoder_length, 1)
        # prev_dec_h = dec_prev_hidden[0].repeat(1,7,1)
        # atten_input = torch.cat([enc_output, prev_dec_h], dim=-1)  # batch*length*3*lstm_size
        # attention_weights = self.attention_fc_2(F.relu(self.attention_fc_1(atten_input)))  # batch_size * length *1的权重矩阵
        # attention_weights = F.softmax(attention_weights, dim=1)  # alpha: batch*length*1
        # atten_output = torch.sum(attention_weights * enc_output, dim=1).unsqueeze(1)# bs *1 *(2*lstm_size)
        #
        # # 复制attention的维度
        # atten_output = atten_output.repeat(1, self.decoder_length, 1)

        # 变化类型
        # atten_output = atten_output.type(torch.DoubleTensor)
        # source_data = source_data.type(torch.DoubleTensor)

        dec_lstm_input = source_data  # bs * 1 *(2*lstm_size+features_size)
        # 变化类型
        # dec_lstm_input = dec_lstm_input.type(torch.FloatTensor)
        # dec_prev_hidden = dec_prev_hidden.type(torch.DoubleTensor)

        dec_output, dec_hidden = self.decoder(dec_lstm_input, dec_prev_hidden)
        # dec_output: bs*1*lstm_size, dec_hidden [bs*1*lstm_size, bs*1*lstm_size]
        #
        return dec_output, dec_hidden

    def forward(self, source_data, target_data):
        #source_data = source_data.type(torch.FloatTensor)  # 这个输入数据 是batchsize*encoder_length*features+1的

        enc_output, enc_hidden = self.encoder(source_data)
        # enc_output.shape: batch_size*length*（2*lstm_size） 只返回所有hidden,并且concat了正向和反向两个
        # enc_hidden：[[h1,h2,h3,h4],[c1,c2,c3,c4]] 返回每层最后一个时间步的h和c

        # 先把attention存起来
        self.atten_outputs = Variable(torch.zeros(target_data.shape[0],
                                                  target_data.shape[1],
                                                  enc_output.shape[2]))  # batch*length* (2*lstm_size) c和h的维度是一样的
        # decoder batch*length*lstm_size
        self.dec_outputs = Variable(torch.zeros(target_data.shape[0], target_data.shape[1], enc_hidden[0].shape[2]))

        # 把h和c都传到LSTM里面去 用作decoder的初始值,加一个维度 就是1*batchsize*lstm_size
        dec_prev_hidden = [enc_hidden[0][0].unsqueeze(0), enc_hidden[1][0].unsqueeze(0)]

        # for i in range(7):
        #         #     # 接下来进行attention_forward,输入是target的features，dec_prev_hidden,enc_output
        #         #     atten_output, dec_output, dec_hidden = self.attention_forward(target_data, dec_prev_hidden, enc_output)
        #         #     self.atten_outputs[:, i] = atten_output.squeeze()
        #         #     self.dec_outputs[:, i] = dec_output.squeeze()
        #         #     dec_prev_hidden = dec_hidden
        dec_output, dec_hidden = self.attention_forward(target_data, dec_prev_hidden, enc_output)

        # bs * length *(features + 2lstm_size+lstm_size)
        # 变化类型
        # target_data = target_data.type(torch.DoubleTensor)
        # atten_output = atten_output.type(torch.DoubleTensor)
        # dec_output = dec_output.type(torch.DoubleTensor)

        class_input = torch.cat([target_data, dec_output], dim=2)
        # 变化类型
        #class_input= torch.tensor(class_input, dtype=torch.float32)




        out = self.fc_1(class_input)
        out = F.relu(out)
        out = self.fc_2(out)
        if self.training is True:
            return out
        else:
            return out