import torch
from transformer import Att_NMT
import torch.nn as nn
import torch.optim as optim
from data_loader import TrainData, TestData
from torch.utils.data import DataLoader
import numpy as np
from model import Loung_NMT
from transformer import NN



def test(model, genTestData, MAPE, device):
    model.eval()
    # 开始测试
    pred = []
    tagt = []
    with torch.no_grad():
        for i, (en, de, tg) in enumerate(genTestData):
            if i == 0:
                out = model(en.to(device), de.to(device))
                start = en
            else:
                ago = start[:, 1:, :]  # 除去第一列
                new = en[:, -1:, :]  # 新的后一列
                start = torch.cat([ago, new], dim=1)
                start[:, -1, -1] = out[0].view(-1)
                out = model(start.to(device), de.to(device))
            pred.append(out[0].item())
            tagt.append(tg.item())
    diff = np.abs(np.array(np.power(10, tagt)) - np.array(np.power(10, pred)))
    _MAPE = np.mean(diff / np.power(10, tagt))
    print('MAPE: ', _MAPE)
    if _MAPE < MAPE:
        MAPE = _MAPE
        # 保存模型
        torch.save(model, "./Best_transformer.pkl")
        print("The model updated...")
    return MAPE






Epoch = 1000
# Model parameters


# Config
device = torch.device("cuda:0" if torch.cuda.is_available() else "cpu")
#device = 'cpu'
print(f"Using device {device}")

# Load transformer with Adam optimizer and MSE loss function
#model = NN().to(device)
#model = Att_NMT(feature_size = 14, addition_feature_size=4, output_dim= 1, encoder_length=24,decoder_length=1,lstm_size=128, training=True).to(device)

model = Loung_NMT(input_feature_size=15, add_feature_size=4, output_size=1,
                     lstm_size=128, score_f="concat", attention_c="local_p", feed_input=True, window_size=14,
                     reverse=True).to(device)


criterion = nn.MSELoss(reduce=True)
# criterion = nn.MSELoss()
optimizer = optim.Adam(model.parameters())

# 加载数据
trainData = TrainData()
genTrainData = DataLoader(trainData, batch_size=32, shuffle=True)
testData = TestData()
genTestData = DataLoader(testData, batch_size=1)


# 训练
MAPE = 1
for ep in range(Epoch):
    loss_mean = 0.
    model.train()
    for i, (feature, add_feature, tg) in enumerate(genTrainData):
        optimizer.zero_grad()
        feature, add_feature, tg = feature.to(device), add_feature.to(device), tg.to(device)
        out,_ = model(feature, add_feature, is_gpu=True)
        m = out.reshape(-1, 1)

        loss = criterion(m, tg)
        #loss += model.nn_kl_divergence()*KL_weighted
        loss.backward()
        # update
        optimizer.step()
        # 打印训练信息
        loss_mean += loss.item()
        if (i + 1) % 10 == 0:
            loss_mean = loss_mean / 10
            print("Training:Epoch[{:0>3}/{:0>3}] Iteration[{:0>3}/{:0>3}] Loss: {:.10f} MSELoss {:.10f} ".format(ep, Epoch, i + 1, len(genTrainData), loss_mean,criterion(m, tg)))
            loss_mean = 0.




    # 测试
    #MAPE = test(model, genTestData, MAPE, device)
    torch.save(model, "./transformer.pkl")