from torch.utils.data import DataLoader
from data_loader import TestData
import torch
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
import networkx as nx


device = torch.device("cuda:0" if torch.cuda.is_available() else "cpu")
# 加载数据
testData = TestData()
genTestData = DataLoader(testData, batch_size=1)
# 加载模型
#model = torch.load('transformer.pkl')
model = torch.load('Best_transformer.pkl')
model.to(device)
model.to(device)
model.eval()
# 开始测试
pred = []
tagt = []
ad_matrix = []

with torch.no_grad():
    for i, (en, de, tg) in enumerate(genTestData):
        change_signal = len(pred)
        if change_signal % 1 == 0:
            en_in = en.transpose(-2, -1)

            out, adp = model(en_in.to(device), idx=None, add_feature=de.to(device), lstm_input=en.to(device))
            start = en
        else:
            ago = start[:, 1:, :]  # 除去第一列
            new = en[:, -1:, :]  # 新的后一列
            start = torch.cat([ago, new], dim=1)
            start[:, -1, -1] = out.view(-1)

            start_in = start.transpose(-2, -1)

            out, adp = model(start_in.to(device), idx=None, add_feature=de.to(device), lstm_input=start.to(device))
        pred.append(out.item())
        tagt.append(tg.item())
        ad_matrix.append(adp.cpu().numpy())

ad_matrix = np.array(ad_matrix)
ad_matrix = ad_matrix.sum(axis=0)
ad_matrix = ad_matrix/6576

G = nx.Graph()
adp = adp.cpu().numpy()
F = nx.from_numpy_array(ad_matrix)
print(F.edges)
print(len(F.edges))
labeldict = {}
labeldict[0] = 'day_of_year'
labeldict[1] = 'week_of_year'
labeldict[2] = 'month'
labeldict[3] = 'hour'
labeldict[4] = 'Pressure_land(hPa)'
labeldict[5] = 'Pressure_sea(hPa)'
labeldict[6] = 'Penetration(mm)'
labeldict[7] = 'Temperature'
labeldict[8] = 'Dew_temperature'
labeldict[9] = 'Water_vapor_pressure(hPa)'
labeldict[10] = 'Humidity(%)'
labeldict[11] = 'Wind speed(m/s)'
labeldict[12] = 'Radation_time(h)'
labeldict[13] = 'Cloud_cover'
labeldict[14] = 'Solar_Radiatiion_modification(MJ/m2)'
colors = ['yellow', 'blue', 'blue', 'green',  'green', 'green',
          'green','green','green','green','green','green','yellow', 'blue', 'red',]
nx.draw(F, labels =labeldict, with_labels = True, node_color=colors)
nx.write_gexf(F,'GAT.gexf')
plt.show()
# 结果保存

result_target = np.power(2, np.array(tagt).squeeze())
result_target = np.where(result_target < 0.012, 0, result_target)

result_pred = np.power(2, np.array(pred).squeeze())
result_pred = np.where(result_pred < 0.012, 0, result_pred)

results = {'target': result_target, 'pred': result_pred }
df = pd.DataFrame(results)
df.to_excel('result.xlsx', index=False)
# 计算MAPE
diff = np.abs(result_target - result_pred)
print('MAPE_indicator: ', np.sum(diff))
# 画图
plt.figure(figsize=(10, 5))
plt.plot(np.array(tagt).squeeze())
plt.plot(np.array(pred).squeeze())
plt.legend(['gnd', 'pred'])
plt.show()